==> cpm-1.0.0.tar.gz: cpm-1.0.0/MPIBlib/src/mpib_utilities.c <==
#include "mpib.h"
#include <math.h>
#include <gsl/gsl_cdf.h>
#include <gsl/gsl_statistics_double.h>
#include <malloc.h>
#include <stdio.h>

double MPIB_ci(double cl, int reps, double* T)
{
	return fabs(gsl_cdf_tdist_Pinv(cl, reps - 1)) * gsl_stats_sd(T, 1, reps) / sqrt(reps); 
}

MPIB_pairs* MPIB_build_pairs(int n)
{
	MPIB_pair* list = NULL;
	MPIB_pair* iter = NULL;
	int i, j;
	for (i = 0; i < n - 1; i++)
	{
		for (j = i + 1; j < n; j++)
		{
			if (!iter)
				(iter = list = (MPIB_pair*)malloc(sizeof(MPIB_pair)))->prev = NULL;
			else
			{
				(iter->next = (MPIB_pair*)malloc(sizeof(MPIB_pair)))->prev = iter;
				iter = iter->next; 
			}
			iter->values[0] = i;
			iter->values[1] = j;
			iter->next = NULL;
		}
	}

	MPIB_pairs* pairs = NULL;
	MPIB_pairs* curr_pairs = NULL;
	while (list)
	{
		if (!curr_pairs)
			(curr_pairs = pairs = (MPIB_pairs*)malloc(sizeof(MPIB_pairs)))->prev = NULL;
		else
		{
			(curr_pairs->next = (MPIB_pairs*)malloc(sizeof(MPIB_pairs)))->prev = curr_pairs;
			curr_pairs = curr_pairs->next;
		}
		curr_pairs->list = NULL;
		curr_pairs->next = NULL;

		MPIB_pair* curr_pair = NULL;
		MPIB_pair* iter1 = list;
		while (iter1)
		{
			int new = 1;
			MPIB_pair* iter2 = curr_pairs->list;
			while (new && iter2)
			{
				int i, j;
				for (i = 0; i < 2; i++)
					for (j = 0; j < 2; j++)
						new &= iter1->values[i] != iter2->values[j];
				iter2 = iter2->next;
			}
			if (new)
			{
				if (iter1 == list)
					list = list->next;
				if (iter1->prev)
					iter1->prev->next = iter1->next;
				if (iter1->next)
					iter1->next->prev = iter1->prev;
				if (!curr_pair)
				{
					curr_pair = curr_pairs->list = iter1;
					curr_pair->prev = NULL;
				}
				else
				{
					(curr_pair->next = iter1)->prev = curr_pair;
					curr_pair = iter1;
				}
				iter1 = iter1->next;
				curr_pair->next = NULL;
			}
			else
				iter1 = iter1->next;
		}
	}
	return pairs;
}

void MPIB_free_pair(MPIB_pair* pair)
{
	if (pair)
	{
		MPIB_free_pair(pair->next);
		free(pair);
	}
}

void MPIB_free_pairs(MPIB_pairs* pairs)
{
	if (pairs)
	{
		MPIB_free_pair(pairs->list);
		MPIB_free_pairs(pairs->next);
		free(pairs);
	}
}

void MPIB_print_precision(MPIB_precision precision)
{
	printf("#min_reps\t%d\n", precision.min_reps); 
	printf("#max_reps\t%d\n", precision.max_reps); 
	printf("#cl\t\t%le\n", precision.cl); 
	printf("#eps\t\t%le\n", precision.eps); 
}

void MPIB_print_p2p_table(int M, MPIB_precision precision, int n, MPIB_result* results)
{
	printf("#msg\t\t%d\n", M);
	MPIB_print_precision(precision);
	printf
	(
"#p2p\t\
time\t\t\
wtick\t\
reps\t\
ci\n"
	);
	int i, index;
	for(i = 0, index = 0; i < n - 1; i++)
	{
		int j;
		for(j = i + 1; j < n; j++, index++)
			printf
			(
"%d-%d\t\
%le\t\
%d\t\
%d\t\
%le\n",
				i, j,
				results[index].T,
				results[index].wtick < results[index].T,
				results[index].reps,
				results[index].ci
			);
	}
	fflush(stdout);
}

void MPIB_print_p2p_th(MPIB_precision precision, int n)
{
	MPIB_print_precision(precision);
	printf("#\t");
	int i, index;
	for(i = 0, index = 0; i < n - 1; i++)
	{
		int j;
		for(j = i + 1; j < n; j++, index++)
			printf("%d-%d\t\t\t\t\t\t", i, j);
	}
	printf("\n#msg\t");
	for(i = 0, index = 0; i < n - 1; i++)
	{
		int j;
		for(j = i + 1; j < n; j++, index++)
			printf
			(
"time\t\t\
wtick\t\
reps\t\
ci\t\t"
			);
	}
	printf("\n");
}

void MPIB_print_p2p_tr(int M, int n, MPIB_result* results)
{
	printf("%d\t", M);
	int i, index;
	for(i = 0, index = 0; i < n - 1; i++)
	{
		int j;
		for(j = i + 1; j < n; j++, index++)
			printf
			(
"%le\t\
%d\t\
%d\t\
%le\t",
				results[index].T,
				results[index].wtick < results[index].T,
				results[index].reps,
				results[index].ci
			);
	}
	printf("\n");
	fflush(stdout);
}

void MPIB_print_coll_th(const char* operation, int n, int root, MPIB_precision precision)
{
	printf("#nodes\t\t%d\n", n);
	printf("#root\t\t%d\n", root);
	MPIB_print_precision(precision);
	printf
	(
"%smsg\t\
%s\t\t\
wtick\t\
reps\t\
ci\n",
		operation == NULL ? "#" : "", 
		operation == NULL ? "time" : operation
	);
}

void MPIB_print_coll_tr(int M, MPIB_result result)
{
	printf
	(
"%d\t\
%le\t\
%d\t\
%d\t\
%le\n",
		M,
		result.T,
		result.wtick < result.T,
		result.reps,
		result.ci
	);
	fflush(stdout);
}

==> cpm-1.0.1.tar.gz: cpm-1.0.1/MPIBlib/src/mpib_utilities.c <==
#include "mpib.h"
#include <math.h>
#include <gsl/gsl_cdf.h>
#include <gsl/gsl_statistics_double.h>
#include <malloc.h>
#include <stdio.h>

double MPIB_ci(double cl, int reps, double* T)
{
	return fabs(gsl_cdf_tdist_Pinv(cl, reps - 1)) * gsl_stats_sd(T, 1, reps) / sqrt(reps); 
}

MPIB_pairs* MPIB_build_pairs(int n)
{
	MPIB_pair* list = NULL;
	MPIB_pair* iter = NULL;
	int i, j;
	for (i = 0; i < n - 1; i++)
	{
		for (j = i + 1; j < n; j++)
		{
			if (!iter)
				(iter = list = (MPIB_pair*)malloc(sizeof(MPIB_pair)))->prev = NULL;
			else
			{
				(iter->next = (MPIB_pair*)malloc(sizeof(MPIB_pair)))->prev = iter;
				iter = iter->next; 
			}
			iter->values[0] = i;
			iter->values[1] = j;
			iter->next = NULL;
		}
	}

	MPIB_pairs* pairs = NULL;
	MPIB_pairs* curr_pairs = NULL;
	while (list)
	{
		if (!curr_pairs)
			(curr_pairs = pairs = (MPIB_pairs*)malloc(sizeof(MPIB_pairs)))->prev = NULL;
		else
		{
			(curr_pairs->next = (MPIB_pairs*)malloc(sizeof(MPIB_pairs)))->prev = curr_pairs;
			curr_pairs = curr_pairs->next;
		}
		curr_pairs->list = NULL;
		curr_pairs->next = NULL;

		MPIB_pair* curr_pair = NULL;
		MPIB_pair* iter1 = list;
		while (iter1)
		{
			int new = 1;
			MPIB_pair* iter2 = curr_pairs->list;
			while (new && iter2)
			{
				int i, j;
				for (i = 0; i < 2; i++)
					for (j = 0; j < 2; j++)
						new &= iter1->values[i] != iter2->values[j];
				iter2 = iter2->next;
			}
			if (new)
			{
				if (iter1 == list)
					list = list->next;
				if (iter1->prev)
					iter1->prev->next = iter1->next;
				if (iter1->next)
					iter1->next->prev = iter1->prev;
				if (!curr_pair)
				{
					curr_pair = curr_pairs->list = iter1;
					curr_pair->prev = NULL;
				}
				else
				{
					(curr_pair->next = iter1)->prev = curr_pair;
					curr_pair = iter1;
				}
				iter1 = iter1->next;
				curr_pair->next = NULL;
			}
			else
				iter1 = iter1->next;
		}
	}
	return pairs;
}

void MPIB_free_pair(MPIB_pair* pair)
{
	if (pair)
	{
		MPIB_free_pair(pair->next);
		free(pair);
	}
}

void MPIB_free_pairs(MPIB_pairs* pairs)
{
	if (pairs)
	{
		MPIB_free_pair(pairs->list);
		MPIB_free_pairs(pairs->next);
		free(pairs);
	}
}

void MPIB_print_processors(MPI_Comm comm)
{
	int rank;
	MPI_Comm_rank(comm, &rank);
	char name[MPI_MAX_PROCESSOR_NAME];
	int len;
	MPI_Get_processor_name(name, &len);
	if (rank == 0)
	{
		printf("#rank\tprocessor\n");
		printf("#%d\t%s\n", rank, name);
		int size;
		MPI_Comm_size(comm, &size);
		int i;
		for (i = 1; i < size; i++)
		{
			MPI_Recv(&rank, 1, MPI_INT, i, 0, comm, MPI_STATUS_IGNORE);
			MPI_Recv(name, MPI_MAX_PROCESSOR_NAME, MPI_CHAR, i, 0, comm, MPI_STATUS_IGNORE);
			printf("#%d\t%s\n", rank, name);
		}
		printf("#\n");
	}
	else
	{
		MPI_Send(&rank, 1, MPI_INT, 0, 0, comm);
		MPI_Send(name, MPI_MAX_PROCESSOR_NAME, MPI_CHAR, 0, 0, comm);
	}
}

void MPIB_print_precision(MPIB_precision precision)
{
	printf("#min_reps\t%d\n", precision.min_reps); 
	printf("#max_reps\t%d\n", precision.max_reps); 
	printf("#cl\t\t%le\n", precision.cl); 
	printf("#eps\t\t%le\n", precision.eps); 
}

void MPIB_print_p2p_table(int M, int parallel, MPIB_precision precision, int n, MPIB_result* results)
{
	printf("#msg\t\t%d\n", M);
	printf("#%s\n", parallel ? "parallel" : "sequential");
	MPIB_print_precision(precision);
	printf
	(
"#p2p\t\
time\t\t\
wtick\t\
reps\t\
ci\n"
	);
	int i, index;
	for(i = 0, index = 0; i < n - 1; i++)
	{
		int j;
		for(j = i + 1; j < n; j++, index++)
			printf
			(
"%d-%d\t\
%le\t\
%d\t\
%d\t\
%le\n",
				i, j,
				results[index].T,
				results[index].wtick < results[index].T,
				results[index].reps,
				results[index].ci
			);
	}
	fflush(stdout);
}

void MPIB_print_p2p_th(int parallel, MPIB_precision precision, int n)
{
	printf("#%s\n", parallel ? "parallel" : "sequential");
	MPIB_print_precision(precision);
	printf("#\t");
	int i, index;
	for(i = 0, index = 0; i < n - 1; i++)
	{
		int j;
		for(j = i + 1; j < n; j++, index++)
			printf("%d-%d\t\t\t\t\t\t", i, j);
	}
	printf("\n#msg\t");
	for(i = 0, index = 0; i < n - 1; i++)
	{
		int j;
		for(j = i + 1; j < n; j++, index++)
			printf
			(
"time\t\t\
wtick\t\
reps\t\
ci\t\t"
			);
	}
	printf("\n");
}

void MPIB_print_p2p_tr(int M, int n, MPIB_result* results)
{
	printf("%d\t", M);
	int i, index;
	for(i = 0, index = 0; i < n - 1; i++)
	{
		int j;
		for(j = i + 1; j < n; j++, index++)
			printf
			(
"%le\t\
%d\t\
%d\t\
%le\t",
				results[index].T,
				results[index].wtick < results[index].T,
				results[index].reps,
				results[index].ci
			);
	}
	printf("\n");
	fflush(stdout);
}

void MPIB_print_coll_th(const char* operation, const char* timing, int n, int root, MPIB_precision precision)
{
	printf("#operation\t%s\n", operation);
	printf("#timing\t\t%s\n", timing);
	printf("#nodes\t\t%d\n", n);
	printf("#root\t\t%d\n", root);
	MPIB_print_precision(precision);
	printf
	(
"#msg\t\
time\t\t\
wtick\t\
reps\t\
ci\n"
	);
}

void MPIB_print_coll_tr(int M, MPIB_result result)
{
	printf
	(
"%d\t\
%le\t\
%d\t\
%d\t\
%le\n",
		M,
		result.T,
		result.wtick < result.T,
		result.reps,
		result.ci
	);
	fflush(stdout);
}
